#!/usr/bin/env python
# -*- coding: utf-8 -*-

# file      : code_gen.py
# project   : basebuildc
# copyright : (C) 2014 by Claude Bulin
#
# Generating a skeleton python from one or more files *. Glade or *. Ui
#
# This file is part of basebuildc project
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; version 3.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA


import os
from datetime import date
from fconfig import FConfig


class codegen_py():

	def __init__(self):
		self.ListeDouble = []
		self.list = []
		self.all_object = []	# [ TYPE_OBJECT, NAME_OBJECT, TYPE_SIGNAL, NAME_SIGNAL ]
		self.list_wind = [
			'GtkAboutDialog',
			'GtkAppChooserDialog',
			'GtkApplicationWindow',
			'GtkColorChooserDialog',
			'GtkDialog',
			'GtkFileChooserDialog',
			'GtkFontChooserDialog',
			'GtkMessageDialog',
			'GtkOffscreenWindow',
			'GtkRecentChooserDialog',
			'GtkWindow'
			]


	#
	# write
	#
	def write_block_control( self, file_object, text_control, text_print, initial_indent=0 ):
		if not text_control in self.ListeDouble:
			self.ListeDouble.append( text_control )
			file_object.write( '\t'*1*initial_indent+text_print + '\n' )
			return True
		return False

	def write_block( self, file_object, text_print, initial_indent=0 ):
		file_object.write( '\t'*1*initial_indent+text_print + '\n' )


	#
	# collect_tags
	#
	def collect_tags(self, pathnamefile_glade):
		self.list[:] = []
		self.all_object[:] = []

		fp = open( pathnamefile_glade,"r" )
		self.list = fp.readlines()
		fp.close()

		exit_list_wind = 0
		obj = []
		global_object = []
		bool_signal = False
		for l in self.list:
			if "<object " in l:
				if bool_signal:
					global_object.append( obj[:] )
				elif obj and bool_signal == False:
					obj.append( '' )
					global_object.append( obj[:] )
				bool_signal = False
				obj[:] = []
				obj.append( l.strip() )

				for ll in self.list_wind:
					if ll in l:
						exit_list_wind += 1
						break
				if exit_list_wind > 1:
					break

			elif "<signal " in l and obj:
				bool_signal = True
				obj.append( l.strip() )

		type_object = ""
		name_object = ""
		type_signal = ""
		for glo in global_object:
			for gl in glo:
				# <object class="GtkWindow" id="wind_claude">
				# <object class="GtkCellRendererText" id="cellrenderertext5"/>
				if "<object " in gl:
					pos1 = gl.find('=')
					pos1 += 2
					pos2 = gl.find(' id')
					pos2 -= 1
					type_object = gl[pos1:pos2]

					pos1 = gl.find("id=")
					pos1 += 4
					pos2 = gl.find('">')
					if pos2 == -1:
						pos2 = gl.find('"/>')
					name_object = gl[pos1:pos2]
				# <signal name="delete-event" handler="on_wind_main_delete_event" swapped="no"/>
				# <signal name="delete-event" handler="on_wind_scan_delete_event" swapped="no"/>
				elif "<signal " in gl:
					pos1 = gl.find('=')
					pos1 += 2
					pos2 = gl.find('" handler')
					type_signal = gl[pos1:pos2]

					pos1 = gl.find('ler=')
					pos1 += 5
					pos2 = gl.find('swapped')
					pos2 -= 2
					self.all_object.append( [ type_object, name_object, type_signal, gl[pos1:pos2] ] )
				elif type_object != "":
					self.all_object.append( [ type_object, name_object, '', '' ] )
			type_object = ""
			name_object = ""
			type_signal = ""

		return( self.all_object )
	#
	#
	#
	def test_objects_file_glade( self, pathnamefile_glade ):
		self.list[:] = []
		self.all_object[:] = []

		fp = open( pathnamefile_glade,"r" )
		self.list = fp.readlines()
		fp.close()

		exit_list_wind = 0
		obj = []
		global_object = []
		bool_signal = False
		for l in self.list:
			if "<object " in l:
				if bool_signal:
					global_object.append( obj[:] )
				elif obj and bool_signal == False:
					obj.append( '' )
					global_object.append( obj[:] )
				bool_signal = False
				obj[:] = []
				obj.append( l.strip() )

				for ll in self.list_wind:
					if ll in l:
						exit_list_wind += 1
						break
				if exit_list_wind > 1:
					break

			elif "<signal " in l and obj:
				bool_signal = True
				obj.append( l.strip() )

		self.list[:] = []
		self.all_object[:] = []

		if exit_list_wind > 1:
			return False

		return True
	#
	#
	#
	def create_code( self, fp, name_glade_file, p_ProjectName ):
		# Fetch collected tags
		tags = self.collect_tags( name_glade_file )

		BaseName = os.path.basename( name_glade_file )
		Name = BaseName.split(".")[0]
		Ext  = BaseName.split(".")[-1]

		list_name_objects = []
		for tg in tags:
# 			if tg[1] not in name_obj:
# 				name_obj.append( tg[1] )
			if tg[1] and tg[3]:
				if tg[0] not in self.list_wind:
					if tg[1] not in list_name_objects:
						list_name_objects.append( tg[1] )
		for no in list_name_objects:
			self.write_block( fp, '// GtkWidget	*%s = NULL;' % no, 1 )

		self.write_block( fp, '', 0 )
		self.write_block( fp, 'if( NULL == (GtkBuilderProjet_%s = Builder_open( "%s", GtkBuilderProjet_%s ))) {' % (Name, BaseName, Name), 1 )
		self.write_block( fp, '	return( EXIT_FAILURE );', 1 )
		self.write_block( fp, '}', 1 )
		for tg in tags:
			self.write_block( fp, '// Add language support from glade file', 1 )
			self.write_block( fp, 'gtk_builder_set_translation_domain(  GtkBuilderProjet_%s, "%s" );' % (Name, tg[1]), 1 )
			break
		self.write_block( fp, 'gtk_builder_connect_signals( GtkBuilderProjet_%s, NULL );' % Name, 1 )
		for tg in tags:
			#if tg[0] == 'GtkWindow':
			self.write_block( fp, '', 0 )
			self.write_block( fp, 'WindMain_%s = GTK_WIDGET( gtk_builder_get_object( GtkBuilderProjet_%s, "%s" ));' % (Name, Name, tg[1]), 1 )
			break
		for no in list_name_objects:
			self.write_block( fp, '// %s=GTK_WIDGET( gtk_builder_get_object( GtkBuilderProjet_%s, "%s" ));' % (no, Name, no), 1 )

		self.write_block( fp, '', 0 )
		self.write_block( fp, 'if( NULL != FromWindMain ) {', 1 )
		self.write_block( fp, '	gtk_window_set_transient_for (GTK_WINDOW(WindMain_%s), GTK_WINDOW(FromWindMain));' % Name, 1 )
		self.write_block( fp, '	gtk_window_set_modal (GTK_WINDOW(WindMain_%s), TRUE);' % Name, 1 )
		self.write_block( fp, '}', 1 )

		self.write_block( fp, '// resize window', 1 )
		self.write_block( fp, '// gtk_window_resize(GTK_WINDOW(WindMain_%s), Config.WinWidth, Config.WinHeight);' % Name, 1 )
		self.write_block( fp, '// place the window', 1 )
		self.write_block( fp, '// gtk_window_move(GTK_WINDOW(WindMain_%s), Config.WinPos_X, Config.WinPos_Y);' % Name, 1 )
		self.write_block( fp, '// icon apply', 1 )
		self.write_block( fp, '// libutils_set_default_icone_to_win( WindMain_%s, "%s.png"  );' % (Name, p_ProjectName), 1 )
		self.write_block( fp, '', 0 )
		self.write_block( fp, 'PackageVersion = g_strdup_printf( "%s  -  %s", PACKAGE_NAME, VERSION );', 1 )
		self.write_block( fp, 'gtk_window_set_title( GTK_WINDOW( WindMain_%s ), PackageVersion );' % Name, 1 )
		self.write_block( fp, 'g_free( PackageVersion );', 1 )
		self.write_block( fp, 'PackageVersion = NULL;', 1 )
		self.write_block( fp, '', 0 )
		self.write_block( fp, '// place your initialization code here', 1 )
		self.write_block( fp, '', 0 )
		self.write_block( fp, 'gtk_widget_show_all( WindMain_%s );' % Name, 1 )
		self.write_block( fp, '', 0 )
		# fp.close()
		tags[:] = []
	#
	# TODO: TO REMOVE
	#
	def create_name_object( self, fp, name_glade_file, p_ProjectName ):
		# Fetch collected tags
		tags = self.collect_tags( name_glade_file )

		name_obj = []
		for tg in tags:
# 			if tg[1] not in name_obj:
# 				name_obj.append( tg[1] )
			if tg[1] and tg[3]:
				if tg[0] not in self.list_wind:
					if tg[1] not in name_obj:
						name_obj.append( tg[1] )

		BaseName = os.path.basename( name_glade_file )
		Name = BaseName.split(".")[0]
		Ext  = BaseName.split(".")[-1]

		for no in name_obj:
			self.write_block( fp, 'GtkWidget	*%s = NULL;' % no, 1 )
		self.write_block( fp, '', 0 )
		for no in name_obj:
			self.write_block( fp, '%s=GTK_WIDGET( gtk_builder_get_object( GtkBuilderProjet_%s, "%s" ));' % (no, Name, no), 1 )
		self.write_block( fp, '', 0 )
		# fp.close()
		tags[:] = []
	#
	#
	#
	def create_callbacks( self, fp, name_glade_file, p_ProjectName ):
		# Fetch collected tags
		tags = self.collect_tags( name_glade_file )

		BaseName = os.path.basename( name_glade_file )
		Name = BaseName.split(".")[0]
		Ext  = BaseName.split(".")[-1]

		self.write_block( fp, 'GtkBuilder	*GtkBuilderProjet_%s = NULL;' % Name, 0 )
		self.write_block( fp, 'GtkWidget	*WindMain_%s = NULL;\n' % Name, 0 )

		self.ListeDouble[:] = []
		for tg in tags:
			bool_type_gboolean = False
			if tg[2] != "":
				if "activate" == tg[2]:
					definition = 'void %s( GtkEntry *editable, gpointer data )' % tg[3]
				elif "button_press_event" == tg[2] or "button-press-event" == tg[2]:
					bool_type_gboolean = True
					definition = 'gboolean %s( GtkWidget *widget, GdkEventButton *event, gpointer user_data )' % tg[3]
				elif "button_release_event" == tg[2] or "button-release-event" == tg[2]:
					definition = 'gboolean %s( GtkWidget *widget, GdkEventButton *event, gpointer user_data )' % tg[3]
					bool_type_gboolean = True
				#
				# TODO
				#
				elif "changed" == tg[2]:
					if tg[0] == 'GtkEntry':
						definition = 'void %s( GtkEditable *editable, gpointer user_data )' % tg[3]
					elif tg[0] == 'GtkComboBox':
						definition = 'void %s( GtkComboBox *combobox, gpointer user_data )' % tg[3]
					elif tg[0] == 'GtkSpinButton':
						definition = 'void %s( GtkSpinButton *spinbutton, gpointer user_data )' % tg[3]
					else:
						definition = '// UNDEFINED: %s %s %s %s' % (tg[0], tg[1], tg[2], tg[3])

				elif "clicked" == tg[2]:
					definition = 'void %s( GtkButton *button, gpointer user_data )' % tg[3]
				elif "close" == tg[2]:
					definition = 'void %s( GtkDialog *Dialog, gpointer data )' % tg[3]
				elif "day_selected" == tg[2] or "day-selected" == tg[2]:
					definition = 'void %s( GtkCalendar *calendar, gpointer user_data )' % tg[3]
				elif "delete_event" == tg[2] or "delete-event" == tg[2]:
					definition = 'gboolean %s( GtkWidget *widget, GdkEvent *event, gpointer user_data )' % tg[3]
					bool_type_gboolean = True
				elif "destroy_event" == tg[2] or "destroy-event" == tg[2]:
					definition = 'gboolean %s( GtkWidget *widget, GdkEvent *event, gpointer user_data )' % tg[3]
					bool_type_gboolean = True
				elif "destroy" == tg[2]:
					definition = 'void %s( GtkObject *object, gpointer user_data )' % tg[3]
				elif "draw" == tg[2]:
					definition = 'gboolean %s( GtkWidget *widget, cairo_t *cr, gpointer user_data )' % tg[3]
					bool_type_gboolean = True
				elif "enter_notify_event" == tg[2] or "enter-notify-event" == tg[2]:
					definition = 'gboolean %s( GtkWidget *widget, GdkEventCrossing *event, gpointer user_data )' % tg[3]
					bool_type_gboolean = True
				elif "event" == tg[2]:
					definition = 'gboolean %s( GtkWidget *widget, GdkEvent *event, gpointer user_data )' % tg[3]
					bool_type_gboolean = True
				elif "event_after" == tg[2] or "event-after" == tg[2]:
					definition = 'void %s( GtkWidget *widget, GdkEvent *event, gpointer user_data )' % tg[3]
				elif "expose_event" == tg[2] or "expose-event" == tg[2]:
					definition = 'gboolean %s( GtkWidget *widget, GdkEventExpose *event, gpointer user_data )' % tg[3]
					bool_type_gboolean = True
				elif "key_press_event" == tg[2] or "key-press-event" == tg[2]:
					definition = 'gboolean %s( GtkWidget *widget, GdkEventKey *kevent, gpointer user_data )' % tg[3]
					bool_type_gboolean = True
				elif "key_release_event" == tg[2] or "key-release-event" == tg[2]:
					 definition = 'gboolean %s( GtkWidget *widget, GdkEventKey *kevent, gpointer user_data )' % tg[3]
					 bool_type_gboolean = True
				elif "leave_notify_event" == tg[2] or "leave-notify-event" == tg[2]:
					definition = 'gboolean %s( GtkWidget *widget, GdkEventCrossing *event, gpointer user_data )' % tg[3]
					bool_type_gboolean = True
				elif "map_event" == tg[2] or "map-event" == tg[2]:
					definition = 'void %s( GtkWidget *widget, gpointer user_data )' % tg[3]
				elif "motion_notify_event" == tg[2] or "motion-notify-event" == tg[2]:
					definition = 'gboolean %s( GtkWidget *drawingarea, GdkEventMotion *event, gpointer user_data )' % tg[3]
					bool_type_gboolean = True
				elif "realize" == tg[2]:
					definition = 'void %s( GtkWidget *widget, gpointer user_data )' % tg[3]
				elif "response" == tg[2]:
					definition = 'void %s( GtkWidget *widget, gpointer user_data )' % tg[3]
				elif "drag_data_received" == tg[2] or "drag-data-received" == tg[2]:
					definition = 'void %s( GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, GtkSelectionData *data, guint info, guint time, gpointer user_data )' % tg[3]
				elif "toggled" == tg[2]:
					definition = 'void %s( GtkToggleButton *togglebutton, gpointer user_data )' % tg[3]
				elif "switch_page" == tg[2] or "switch-page" == tg[2]:
					definition = 'void %s( GtkNotebook *notebook, gpointer page, guint page_num, gpointer user_data )' % tg[3]
				elif "value_changed" == tg[2] or "value-changed" == tg[2]:
					definition = 'void %s( GtkSpinButton *spinbutton, gpointer user_data )' % tg[3]
				elif "window_state_event" == tg[2] or "window-state-event" == tg[2]:
					definition = 'gboolean %s( GtkWidget *widget, GdkEvent *event, gpointer user_data )' % tg[3]
					bool_type_gboolean = True
				else:
					self.write_block( fp, '// UNDEFINED : CLASS_??? :  StructSignal->Signal = %s' % tg[2], 0 )
					definition = 'void %s( GtkWidget *widget, gpointer user_data )' % tg[3]

				if definition != "" and self.write_block_control( fp, tg[3] , definition, initial_indent=0 ):
					self.write_block( fp, '{', 0 )
					if "delete-event" in tg[2] or "destroy-event" in tg[2]:
						self.write_block( fp, 'gtk_main_quit();', 1 )
						self.write_block( fp, '// gtk_widget_destroy( WindMain_%s );' % Name, 1 )
						self.write_block( fp, '// WindMain_%s = NULL;' % Name, 1 )
						self.write_block( fp, '// GtkBuilderProjet_%s = NULL;' % Name, 1 )
					if bool_type_gboolean == True:
						self.write_block( fp, 'return( FALSE );', 1 )

					if 'GtkTextView' == tg[0] and 'realize' == tg[2]:
						self.write_block( fp, '// SIGNAL_CHANGED', 1 )
						self.write_block( fp, '// void changed_textview (GtkTextBuffer *textbuffer, gpointer user_data) { }', 1 )
						self.write_block( fp, '// IN REALIZE:', 1 )
						self.write_block( fp, '// g_signal_connect((gpointer) gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget)), \"changed\", G_CALLBACK (changed_textview), NULL);', 1 )

						self.write_block( fp, '// SIGNAL_CHANGED', 1 )
						self.write_block( fp, '// void changed_textview (GtkTextBuffer *textbuffer, gpointer user_data) { }', 1 )
						self.write_block( fp, '// IN REALIZE:', 1 )
						self.write_block( fp, '// g_signal_connect((gpointer) gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget)), \"changed\", G_CALLBACK (changed_textview), NULL);', 1 )

					self.write_block( fp, '}', 0 )
					self.write_block( fp, '', 0 )
		# fp.close()
		tags[:] = []
		self.ListeDouble[:] = []












